﻿Imports System.Drawing.Drawing2D

Public Class SlotBar

    Friend Shared LeftTextSize As Single
    Friend Selected As Boolean = False
    Private WithEvents pbox As PictureBox = New PictureBox()
    Private m_graphics As Graphics
    Private m_fillbrush As LinearGradientBrush
    Private m_backbrush As Brush
    Private m_width As Int32
    Private m_height As Int32
    Private m_strings As Boolean
    Private m_max As Single
    Private m_min As Single
    Private m_colors As Int32
    Private m_slot As Int32
    Private m_text As String
    Private m_format As String
    Private m_stringvalue As String = ""
    Private m_value As Single
    Private m_stepmin As Single
    Private m_MousePressed As Boolean
    Private m_RightMousePressed As Boolean
    Private m_eventsenabled As Boolean = False
    Private Shared m_textfontH As Font = New Font("Tahoma", 9, FontStyle.Bold, GraphicsUnit.Point)
    Private Shared m_textfontFixed As Font = New Font("Courier New", 10, FontStyle.Bold, GraphicsUnit.Point)
    'Private Shared m_textfontFixed As Font = New Font("Lucida Console", 10, FontStyle.Bold, GraphicsUnit.Point)

    Private m_Bordercolor As Color = Color.FromArgb(160, 160, 140)

    Friend Sub New(ByRef f As Form)
        pbox.Parent = f
        pbox.BackColor = Color.FromArgb(50, 60, 70)
        pbox.Anchor = AnchorStyles.Left Or AnchorStyles.Top
        f.Controls.Add(pbox)
    End Sub

    Friend Sub Destroy()
        If pbox.Parent Is Nothing Then Return
        pbox.Parent.Controls.Remove(pbox)
    End Sub

    ' ====================================================================================================================
    '   SET ASPECT
    ' ====================================================================================================================
    Friend Sub SetAspectParams(ByVal x As Int32, _
                               ByVal y As Int32, _
                               ByVal w As Int32, _
                               ByVal h As Int32, _
                               ByVal strings As Boolean, _
                               ByVal max As Single, _
                               ByVal min As Single, _
                               ByVal colors As Int32, _
                               ByVal nDecimals As Int32)
        m_eventsenabled = False
        m_strings = strings
        m_max = max
        m_min = min
        '
        m_stepmin = NearestPowerOfTen_PosOnly(Math.Abs(m_max - m_min) / 1000.0F)
        ' ------------------------------------------------------------ 9.99 instead of 10 otherwise sometimes 10 < 10 is true
        If Math.Abs(m_max - m_min) / (m_stepmin * 100) < 9.99 Then
            m_stepmin /= 10
        End If
        If m_stepmin = 0 Then m_stepmin = 0.000001
        '
        m_colors = colors
        m_format = "0." + StrDup(nDecimals, "0")
        pbox.Location = New Point(x, y)
        pbox.Width = w
        pbox.Height = h
        m_width = pbox.ClientSize.Width
        m_height = pbox.ClientSize.Height
        SetAspect()
        m_eventsenabled = True
    End Sub

    Private Sub pbox_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles pbox.Resize
        If Not m_eventsenabled Then Return
        SetAspect()
    End Sub

    Private Sub SetAspect()
        InitPictureboxImage(pbox)
        If pbox.Image Is Nothing Then Return
        m_graphics = Graphics.FromImage(pbox.Image)
        If m_strings Then
            Select Case m_colors
                Case 1
                    m_backbrush = New SolidBrush(pbox.BackColor)
                    m_fillbrush = CreateFillBrush(pbox, 180, 1)
                Case 2
                    m_backbrush = New SolidBrush(pbox.BackColor)
                    m_fillbrush = CreateFillBrush(pbox, 180, 4)
                Case 3
                    m_backbrush = New SolidBrush(Color.FloralWhite)
                    m_fillbrush = CreateFillBrush(pbox, 90, 2)
                Case Else
                    m_backbrush = New SolidBrush(Color.FloralWhite)
                    m_fillbrush = CreateFillBrush(pbox, 90, 3)
            End Select
        Else
            m_backbrush = New SolidBrush(pbox.BackColor)
            Select Case m_colors
                Case 1 : m_fillbrush = CreateFillBrush(pbox, 180, 1)
                Case 2 : m_fillbrush = CreateFillBrush(pbox, 90, 2)
                Case 3 : m_fillbrush = CreateFillBrush(pbox, 90, 3)
                Case Else
                    m_backbrush = New SolidBrush(Color.FloralWhite)
                    m_fillbrush = CreateFillBrush(pbox, 90, 4)
            End Select
        End If
        m_graphics.DrawLine(New Pen(m_Bordercolor), 0, 0, m_width, 0)
        m_graphics.FillRectangle(m_backbrush, 1, 2, m_width - 2, m_height - 3)
    End Sub

    ' ====================================================================================================================
    '   BAR PROPERTIES
    ' ====================================================================================================================
    Friend Sub SetSlotIndex(ByVal slot As Int32)
        m_slot = slot
    End Sub

    Friend Function GetSlotIndex() As Int32
        Return m_slot
    End Function

    Friend Sub SetText(ByVal s As String)
        m_text = s
    End Sub

    Private m_Locked As Boolean
    Friend Sub SetLockedStatus(ByVal locked As Boolean)
        m_Locked = locked
    End Sub

    Private Sub SetBorder()
        Dim p As Pen
        If m_strings Then
            If Selected Then
                p = New Pen(Color.Yellow)
            Else
                p = New Pen(pbox.BackColor)
            End If
        Else
            If Selected Then
                If m_colors = 0 Then
                    p = New Pen(Color.Red)
                Else
                    p = New Pen(Color.Yellow)
                End If
            Else
                p = New Pen(pbox.BackColor)
            End If
        End If
        m_graphics.DrawLine(p, 0, 1, m_width, 1)
        m_graphics.DrawLine(p, 0, m_height - 1, m_width, m_height - 1)
    End Sub

    Friend Sub SetStringValue(ByVal stringvalue As String)
        If stringvalue <> m_stringvalue Then
            m_stringvalue = stringvalue
            ShowValue()
        End If
    End Sub

    Friend Sub SetValue(ByVal value As Single)
        If Single.IsNaN(value) AndAlso Single.IsNaN(m_value) Then Return
        If value <> m_value Then
            m_value = value
            ShowValue()
        End If
    End Sub

    Friend Sub DeltaValue(ByVal delta As Single)
        Dim v As Single
        ' -------------------------------------------------- rounding errors corrected with decimal calculations
        'v = CSng(m_value + delta * m_stepmin)
        v = CSng(CDec(m_value) + CDec(delta * m_stepmin))
        ' --------------------------------------------------
        If v < m_min Then v = m_min
        If v > m_max Then v = m_max
        Slots.WriteSlot(m_slot, v)
    End Sub

    ' TODO - EDIT TEXT
    ' ====================================================================================================================
    '   EDIT TEXT
    ' ====================================================================================================================
    Private Sub pbox_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles pbox.Click
        Form2.Hide()
    End Sub

    Private Sub pbox_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles pbox.DoubleClick
        If Not m_strings Then Return
        Dim p As Point = New Point(pbox.ClientRectangle.Left + 200, pbox.ClientRectangle.Top + m_height + 4)
        Form2.Location = pbox.PointToScreen(p)
        Form2.Hide()
        Form2.SetSlot(m_slot)
        Form2.Show(pbox)
    End Sub

    Friend Sub EditText(ByVal k As System.Windows.Forms.Keys)
        If Not m_strings Then Return
        Select Case k
            Case Keys.Back
                If m_stringvalue.Length > 0 Then
                    m_stringvalue = Left(m_stringvalue, m_stringvalue.Length - 1)
                End If
            Case Keys.Delete

            Case Else
                If k >= Keys.A And k <= Keys.Z Then

                    m_stringvalue += k.ToString
                End If
        End Select
        Theremino_Strings.StringSlots.WriteString(m_slot, m_stringvalue)
        ShowValue()
    End Sub


    ' ====================================================================================================================
    '   MOUSE
    ' ====================================================================================================================
    Private StartPoint As Point
    Private Sub pbox_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles pbox.MouseDown
        If m_Locked Then Return
        If m_strings Then
            If Not Me.Selected Then
                Form1.SetSelectedSlotBar(Me)
            End If
        Else
            Cursor.Clip = pbox.RectangleToScreen(pbox.ClientRectangle)
            m_MousePressed = True
            m_RightMousePressed = e.Button = MouseButtons.Right
            '
            StartPoint = e.Location
            FollowCursor(e)
        End If
    End Sub
    Private Sub pbox_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles pbox.MouseMove
        If m_Locked Then Return
        If m_MousePressed Then
            FollowCursor(e)
        End If
    End Sub
    Private Sub pbox_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles pbox.MouseUp
        If m_Locked Then Return
        m_MousePressed = False
        Cursor.Clip = Nothing
        WaitMovement = False
    End Sub

    Private WaitMovement As Boolean = False
    Private Sub FollowCursor(ByVal e As System.Windows.Forms.MouseEventArgs)
        Dim p As Point = e.Location
        ' ------------------------------------------------- test if unselected
        If Not Me.Selected Then
            Form1.SetSelectedSlotBar(Me)
            If e.Button = MouseButtons.Left Then
                WaitMovement = True
                Return
            End If
        End If
        ' ------------------------------------------------- if unselected then wait movement
        If WaitMovement Then
            Dim dist As Double
            dist = Math.Abs(p.X - StartPoint.X) + Math.Abs(p.Y - StartPoint.Y)
            If dist < 15 Then Return
            WaitMovement = False
        End If
        ' ------------------------------------------------- compute value
        Dim n As Single
        n = m_min + (m_max - m_min) * p.X / (m_width - 1.0F)
        If n < m_min Then n = m_min
        If n > m_max Then n = m_max
        ' ------------------------------------------------- right mouse button rounds to 1 / 10 / 100
        If m_RightMousePressed Then
            ' --------------------------------- debug
            'Form1.Text = m_stepmin.ToString
            ' ---------------------------------------
            n /= m_stepmin
            If My.Computer.Keyboard.ShiftKeyDown Then
                n = CInt(n / 1000) * 1000
            ElseIf My.Computer.Keyboard.AltKeyDown Then
                n = CInt(n)
            ElseIf My.Computer.Keyboard.CtrlKeyDown Then
                n = CInt(n / 10) * 10
            Else
                n = CInt(n / 100) * 100
            End If
            n *= m_stepmin
        End If
            Slots.WriteSlot(m_slot, n)
    End Sub

    ' ====================================================================================================================
    '  SHOW BAR
    ' ====================================================================================================================
    Friend Sub ShowValue()
        ' ------------------------------------------------------------ font sizes
        SetFontSizes()
        ' ------------------------------------------------------------ 
        Dim s As String
        Dim sz As SizeF
        Dim norm As Single
        If m_graphics Is Nothing Then Return
        If m_fillbrush Is Nothing Then Return
        ' ------------------------------------------------------------ prepare value as string
        If m_strings Then
            s = m_stringvalue
        Else
            If Single.IsNaN(m_value) Then
                norm = 0
                s = NAN_GetName(m_value)
            Else
                norm = (m_value - m_min) / (m_max - m_min)
                If norm > 1 Then norm = 1
                If norm < 0 Then norm = 0
                s = m_value.ToString(m_format, Globalization.CultureInfo.InvariantCulture)
            End If
        End If
        ' ------------------------------------------------------------ set selected border
        SetBorder()
        ' ------------------------------------------------------------ clear background
        Dim x As Single = m_width * norm + 1
        Dim y As Single = m_height * 0.08F

        If Selected AndAlso m_strings AndAlso (m_colors = 0 Or m_colors = 3) Then
            m_graphics.FillRectangle(Brushes.Orange, 1, 2, x - 1, m_height - 3)
            m_graphics.FillRectangle(Brushes.Yellow, x, 2, m_width - 1, m_height - 3)
        ElseIf Selected AndAlso Not m_strings AndAlso m_colors = 0 Then
            m_graphics.FillRectangle(Brushes.Orange, 1, 2, x - 1, m_height - 3)
            m_graphics.FillRectangle(Brushes.Yellow, x, 2, m_width - 1, m_height - 3)
        Else
            m_graphics.FillRectangle(m_fillbrush, 1, 2, x - 1, m_height - 3)
            m_graphics.FillRectangle(m_backbrush, x, 2, m_width - 1, m_height - 3)
        End If
        ' ------------------------------------------------------------
        If m_strings Then
            ' -------------------------------------------------------- string slots
            Dim TextBrush As Brush
            If m_colors = 1 Then
                TextBrush = Brushes.Yellow
            ElseIf m_colors = 2 Then
                TextBrush = Brushes.Yellow
            ElseIf m_colors = 3 Then
                TextBrush = Brushes.Gray
            Else
                TextBrush = Brushes.Black
            End If
            ' -------------------------------------------------------- 
            m_graphics.DrawString(m_slot.ToString, m_textfontH, TextBrush, 2, y)
            If m_text IsNot Nothing Then
                m_graphics.DrawString(m_text, m_textfontFixed, TextBrush, 12 * y + 8, y * 3.2F - 4)
            End If
            ' -------------------------------------------------------- 
            Dim lts As Single = 12 * y + 8 + LeftTextSize + 20
            If m_colors = 1 Then
                m_graphics.DrawString(s, m_textfontH, m_fillbrush, lts, y)
            ElseIf m_colors = 2 Then
                m_graphics.DrawString(s, m_textfontH, Brushes.Yellow, lts, y)
            ElseIf m_colors = 3 Then
                m_graphics.DrawString(s, m_textfontH, Brushes.Gray, lts, y)
            Else
                m_graphics.DrawString(s, m_textfontH, Brushes.Black, lts, y)
            End If
        Else
            ' -------------------------------------------------------- numeric slots
            Dim TextBrush As Brush
            If m_colors = 0 Then
                If Selected Then
                    TextBrush = Brushes.Black
                Else
                    TextBrush = Brushes.Gray
                End If
            Else
                If Selected Then
                    TextBrush = Brushes.Orange
                Else
                    TextBrush = Brushes.Yellow
                End If
            End If

            ' -------------------------------------------------------- 
            m_graphics.DrawString(m_slot.ToString, m_textfontH, TextBrush, 2, y)
            If m_text IsNot Nothing Then
                m_graphics.DrawString(m_text, m_textfontFixed, TextBrush, 12 * y + 8, y * 3.2F - 4)
            End If
            ' -------------------------------------------------------- 
            sz = TextRenderer.MeasureText(s, m_textfontH)
            m_graphics.DrawString(s, m_textfontH, TextBrush, m_width - sz.Width, y)
        End If
        '
        pbox.Refresh()
    End Sub

    Friend Sub SetFontSizes()
        Dim FontSizeH As Single
        Dim FontSizeF As Single
        '
        FontSizeH = pbox.Height * 0.43F + 2
        FontSizeF = pbox.Height * 0.46F + 2
        '
        If FontSizeH <> m_textfontH.Size Then
            m_textfontH = New Font("Tahoma", FontSizeH, FontStyle.Bold)
        End If
        If FontSizeF <> m_textfontFixed.Size Then
            m_textfontFixed = New Font("Courier New", FontSizeF, FontStyle.Bold)
        End If
    End Sub

    Friend Sub UpdateLeftTextSize()
        Dim s As Single = TextRenderer.MeasureText(m_text, m_textfontFixed).Width
        If s > LeftTextSize Then LeftTextSize = s
    End Sub

End Class
