﻿Imports System.Drawing.Drawing2D

Module Module_Utils

    Friend Sub InitPictureboxImage(ByVal pbox As PictureBox)
        With pbox
            If .ClientSize.Width < 1 Or .ClientSize.Height < 1 Then Return
            If .Image Is Nothing OrElse .Image.Width <> .ClientSize.Width _
                                 OrElse .Image.Height <> .ClientSize.Height Then
                .Image = New Bitmap(.ClientSize.Width, _
                                    .ClientSize.Height, _
                                    Imaging.PixelFormat.Format24bppRgb)
            End If
        End With
    End Sub

    Friend Function CreateFillBrush(ByVal pbox As PictureBox, _
                                    ByVal angle As Single, _
                                    ByVal blend As Int32) As LinearGradientBrush
        If pbox.ClientRectangle.Width < 1 Or _
           pbox.ClientRectangle.Height < 1 Then
            Return Nothing
        End If
        CreateFillBrush = New LinearGradientBrush(pbox.ClientRectangle, _
                                                  Color.Black, _
                                                  Color.Black, _
                                                  angle)
        Dim myBlend As ColorBlend = New ColorBlend()
        Select Case blend
            Case 1
                myBlend.Positions = New Single() {0.0F, 0.4F, 0.5F, 0.6F, 1.0F}
                myBlend.Colors = New Color() {Color.FromArgb(255, 0, 0), _
                                              Color.FromArgb(255, 230, 0), _
                                              Color.FromArgb(250, 250, 0), _
                                              Color.FromArgb(230, 230, 0), _
                                              Color.FromArgb(0, 100, 0)}

            Case 2
                myBlend.Positions = New Single() {0.0F, 0.4F, 0.5F, 0.9F, 1.0F}
                myBlend.Colors = New Color() {Color.FromArgb(200, 250, 150), _
                                              Color.FromArgb(80, 150, 100), _
                                              Color.FromArgb(80, 150, 0), _
                                              Color.FromArgb(80, 150, 0), _
                                              Color.FromArgb(250, 0, 0)}
            Case 3
                myBlend.Positions = New Single() {0.0F, 0.4F, 0.5F, 0.9F, 1.0F}
                myBlend.Colors = New Color() {Color.FromArgb(205, 205, 205), _
                                              Color.FromArgb(40, 150, 150), _
                                              Color.FromArgb(0, 150, 150), _
                                              Color.FromArgb(0, 120, 120), _
                                              Color.FromArgb(0, 80, 80)}
            Case Else
                myBlend.Positions = New Single() {0.0F, 1.0F}
                myBlend.Colors = New Color() {Color.FromArgb(240, 255, 240), _
                                              Color.FromArgb(150, 200, 100)}
        End Select
        CreateFillBrush.InterpolationColors = myBlend
    End Function

    'Friend Function ExtractFirstWord(ByVal s As String) As String
    '    If s = Nothing Then Return ""
    '    s = s.Replace(vbTab, " ").Trim
    '    Dim i As Int32 = s.IndexOf(" ")
    '    If i < 0 Then Return s
    '    Return s.Substring(0, i)
    'End Function
    Friend Function RemoveComments(ByVal s As String) As String
        Dim i As Int32 = s.IndexOf("'")
        If i > 0 Then s = s.Remove(i).TrimEnd
        If i = 0 Then s = ""
        Return s
    End Function


    ' =========================================================================
    '   "SUSPEND REDRAW" HELPER
    ' =========================================================================
    Private Const WM_SETREDRAW As Int32 = 11
    Public Sub SuspendRedraw(ByVal cnt As Control)
        Dim msgSuspendUpdate As Message
        msgSuspendUpdate = Message.Create(cnt.Handle, WM_SETREDRAW, IntPtr.Zero, IntPtr.Zero)
        NativeWindow.FromHandle(cnt.Handle).DefWndProc(msgSuspendUpdate)
    End Sub
    Public Sub ResumeRedraw(ByVal cnt As Control)
        Dim msgResumeUpdate As Message
        msgResumeUpdate = Message.Create(cnt.Handle, WM_SETREDRAW, New IntPtr(1), IntPtr.Zero)
        NativeWindow.FromHandle(cnt.Handle).DefWndProc(msgResumeUpdate)
        cnt.Invalidate()
    End Sub

    ' ========================================================
    '   PROCESS START AND WAIT
    ' ========================================================
    Friend Sub ProcessStartAndWait(ByVal fname As String)
        If Not My.Computer.FileSystem.FileExists(fname) Then Return
        Dim p As Process = Process.Start(fname)
        Do
            System.Threading.Thread.Sleep(100)
        Loop Until p.HasExited
    End Sub

    ' ==============================================================
    '  REPLACE MULTIPLE SPACES and TABS and TRIM  
    '  (3..5 uS for Strings with 32 chars)
    ' ============================================================== 
    Friend Function ReplaceMultipleSpacesAndTabAndTrim(ByVal s As String) As String
        If s = "" Then Return s
        s = s.Replace(vbTab, " ")
        While s.Contains("  ")
            s = s.Replace("  ", " ")
        End While
        Return s.Trim
    End Function

    ' ==============================================================
    '  NEAREST POWER OF TEN (positive numbers only)
    ' ============================================================== 
    Friend Function NearestPowerOfTen_PosOnly(ByVal v As Single) As Single
        Return CSng(Math.Pow(10, Math.Round(Math.Log10(v))))
    End Function

    ' ==============================================================
    '  NEAREST POWER OF TEN (negative numbers too)
    ' ============================================================== 
    Friend Function NearestPowerOfTen_PosNeg(ByVal v As Single) As Single
        Return Math.Sign(v) * CSng(Math.Pow(10, Math.Round(Math.Log10(Math.Abs(v)))))
    End Function


    ' ==============================================================
    '  Test SlotFile and reload it if changed
    ' ============================================================== 
    Friend Sub TestSlotFile()
        ' -------------------------------------------------- reduce the test frequency if called by a fast timer
        Static counter As Int32
        counter += 1
        If counter < 20 Then Return ' 20 = 300 mS (with a 60 Hz Timer)
        counter = 0
        ' --------------------------------------------------  
        Static SlotFileInfo As IO.FileInfo = Nothing
        Dim newinfo As IO.FileInfo = My.Computer.FileSystem.GetFileInfo(GetSlotFilePathAndName)
        If SlotFileInfo Is Nothing Then
            SlotFileInfo = newinfo
        End If
        If newinfo.LastWriteTime <> SlotFileInfo.LastWriteTime Then
            SlotFileInfo = newinfo
            LoadSlotNames()
            Form1.SlotBars_Show()
        End If
    End Sub

   
End Module
